@@ -42,6 +42,11 @@ class BytesTests(EqualityTestsMixin,
         from zope.schema.interfaces import IBytes
         return IBytes
 
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
     def test_validate_wrong_types(self):
         field = self._makeOne()
         self.assertAllRaiseWrongType(
@@ -84,11 +89,15 @@ class BytesTests(EqualityTestsMixin,
         self.assertRaises(UnicodeEncodeError, byt.fromUnicode, u'\x81')
 
     def test_fromUnicode_hit(self):
-
         byt = self._makeOne()
         self.assertEqual(byt.fromUnicode(u''), b'')
         self.assertEqual(byt.fromUnicode(u'DEADBEEF'), b'DEADBEEF')
 
+    def test_fromBytes(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b''), b'')
+        self.assertEqual(field.fromBytes(b'DEADBEEF'), b'DEADBEEF')
+
 
 class ASCIITests(EqualityTestsMixin,
                  WrongTypeTestsMixin,
@@ -150,6 +159,11 @@ class BytesLineTests(EqualityTestsMixin,
         from zope.schema.interfaces import IBytesLine
         return IBytesLine
 
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
     def test_validate_wrong_types(self):
         field = self._makeOne()
         self.assertAllRaiseWrongType(
@@ -166,7 +180,6 @@ class BytesLineTests(EqualityTestsMixin,
             object())
 
     def test_validate_not_required(self):
-
         field = self._makeOne(required=False)
         field.validate(None)
         field.validate(b'')
@@ -183,7 +196,6 @@ class BytesLineTests(EqualityTestsMixin,
         self.assertRaises(RequiredMissing, field.validate, None)
 
     def test_constraint(self):
-
         field = self._makeOne()
         self.assertEqual(field.constraint(b''), True)
         self.assertEqual(field.constraint(b'abc'), True)
@@ -191,6 +203,11 @@ class BytesLineTests(EqualityTestsMixin,
         self.assertEqual(field.constraint(b'\xab\xde'), True)
         self.assertEqual(field.constraint(b'abc\ndef'), False)
 
+    def test_fromBytes(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b''), b'')
+        self.assertEqual(field.fromBytes(b'DEADBEEF'), b'DEADBEEF')
+
 
 class ASCIILineTests(EqualityTestsMixin,
                      WrongTypeTestsMixin,
@@ -832,8 +849,12 @@ class URITests(EqualityTestsMixin,
         from zope.schema.interfaces import IURI
         return IURI
 
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
     def test_validate_wrong_types(self):
-        from zope.schema.interfaces import WrongType
         from zope.schema._compat import non_native_string
         field = self._makeOne()
         self.assertAllRaiseWrongType(
@@ -893,6 +914,51 @@ class URITests(EqualityTestsMixin,
                           field.fromUnicode, u'http://example.com/\nDAV:')
 
 
+class PythonIdentifierTests(EqualityTestsMixin,
+                            WrongTypeTestsMixin,
+                            unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.schema._field import PythonIdentifier
+        return PythonIdentifier
+
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
+    def _getTargetInterface(self):
+        from zope.schema.interfaces import IPythonIdentifier
+        return IPythonIdentifier
+
+    def test_fromUnicode_empty(self):
+        pi = self._makeOne()
+        self.assertEqual(pi.fromUnicode(u''), '')
+
+    def test_fromUnicode_normal(self):
+        pi = self._makeOne()
+        self.assertEqual(pi.fromUnicode(u'normal'), 'normal')
+
+    def test_fromUnicode_strips_ws(self):
+        pi = self._makeOne()
+        self.assertEqual(pi.fromUnicode(u'   '), '')
+        self.assertEqual(pi.fromUnicode(u' normal  '), 'normal')
+
+    def test__validate_miss(self):
+        from zope.schema.interfaces import InvalidValue
+        pi = self._makeOne()
+        with self.assertRaises(InvalidValue) as exc:
+            pi._validate('not-an-identifier')
+
+        ex = exc.exception
+        self.assertIs(ex.field, pi)
+        self.assertEqual(ex.value, 'not-an-identifier')
+
+    def test__validate_hit(self):
+        pi = self._makeOne()
+        pi._validate('is_an_identifier')
+
+
 class DottedNameTests(EqualityTestsMixin,
                       WrongTypeTestsMixin,
                       unittest.TestCase):
@@ -905,6 +971,11 @@ class DottedNameTests(EqualityTestsMixin,
         from zope.schema.interfaces import IDottedName
         return IDottedName
 
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
     def test_ctor_defaults(self):
         dotted = self._makeOne()
         self.assertEqual(dotted.min_dots, 0)
@@ -990,6 +1061,12 @@ class DottedNameTests(EqualityTestsMixin,
         field = self._makeOne()
         self.assertEqual(field.fromUnicode(u'dotted.name'), 'dotted.name')
 
+        # Underscores are allowed in any component
+        self.assertEqual(field.fromUnicode(u'dotted._name'), 'dotted._name')
+        self.assertEqual(field.fromUnicode(u'_leading_underscore'), '_leading_underscore')
+        self.assertEqual(field.fromUnicode(u'_dotted.name'), '_dotted.name')
+        self.assertEqual(field.fromUnicode(u'_dotted._name'), '_dotted._name')
+
     def test_fromUnicode_invalid(self):
         from zope.schema.interfaces import ConstraintNotSatisfied
         from zope.schema.interfaces import InvalidDottedName
@@ -1018,8 +1095,12 @@ class IdTests(EqualityTestsMixin,
         from zope.schema.interfaces import IId
         return IId
 
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
     def test_validate_wrong_types(self):
-        from zope.schema.interfaces import WrongType
         from zope.schema._compat import non_native_string
         field = self._makeOne()
         self.assertAllRaiseWrongType(
@@ -1459,8 +1540,6 @@ class SetTests(WrongTypeTestsMixin,
         self.assertTrue(self._makeOne().unique)
 
     def test_validate_wrong_types(self):
-        from zope.schema.interfaces import WrongType
-
         field = self._makeOne()
         self.assertAllRaiseWrongType(
             field,
@@ -1510,8 +1589,6 @@ class MappingTests(EqualityTestsMixin,
         self.assertRaises(ValueError, self._makeOne, value_type=object())
 
     def test_validate_wrong_types(self):
-        from zope.schema.interfaces import WrongType
-
         field = self._makeOne()
         self.assertAllRaiseWrongType(
             field,
@@ -1676,6 +1753,63 @@ class DictTests(MutableMappingTests):
         with self.assertRaises(WrongType):
             super(DictTests, self).test_mutable_mapping()
 
+class NativeStringTests(EqualityTestsMixin,
+                        WrongTypeTestsMixin,
+                        unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.schema._field import NativeString
+        return NativeString
+
+    def _getTargetInterface(self):
+        from zope.schema.interfaces import INativeString
+        return INativeString
+
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
+    def test_fromBytes(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b''), '')
+        self.assertEqual(field.fromBytes(b'DEADBEEF'), 'DEADBEEF')
+
+    def test_fromUnicode(self):
+        field = self._makeOne()
+        self.assertIsInstance(field.fromUnicode(u''), str)
+        self.assertEqual(field.fromUnicode(u''), '')
+        self.assertEqual(field.fromUnicode(u'DEADBEEF'), 'DEADBEEF')
+
+
+class NativeStringLineTests(EqualityTestsMixin,
+                            WrongTypeTestsMixin,
+                            unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.schema._field import NativeStringLine
+        return NativeStringLine
+
+    def _getTargetInterface(self):
+        from zope.schema.interfaces import INativeStringLine
+        return INativeStringLine
+
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
+    def test_fromBytes(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b''), '')
+        self.assertEqual(field.fromBytes(b'DEADBEEF'), 'DEADBEEF')
+
+    def test_fromUnicode(self):
+        field = self._makeOne()
+        self.assertIsInstance(field.fromUnicode(u''), str)
+        self.assertEqual(field.fromUnicode(u''), '')
+        self.assertEqual(field.fromUnicode(u'DEADBEEF'), 'DEADBEEF')
+
 
 def _makeSampleVocabulary():
     from zope.interface import implementer
